package gs

import java.nio.charset.StandardCharsets
import java.nio.file.Files
import java.nio.file.Paths
import sbt._

object CalVerPlugin extends AutoPlugin {
  override def trigger = allRequirements

  val autoImport = CalVerKeys

  import autoImport._

  val DefaultOutputFile: String = ".version"

  lazy val calVerDefaults: Seq[Setting[_]] =
    Seq(
      calVerReleasePattern    := Some(CalVer.Defaults.ReleasePattern),
      calVerPreReleasePattern := Some(CalVer.Defaults.PreReleasePattern),
      calVerOutputFile        := Some(DefaultOutputFile)
    )

  override lazy val projectSettings: Seq[Setting[_]] = {
    val isRelease = PluginProperties.isRelease()

    Seq(
      calVerPattern := {
        if (isRelease)
          calVerReleasePattern.value.getOrElse(CalVer.Defaults.ReleasePattern)
        else
          calVerPreReleasePattern.value.getOrElse(
            CalVer.Defaults.PreReleasePattern
          )
      },
      calVer := CalVer.render(calVerPattern.value).value,
      calVerInfo := {
        val log = Keys.streams.value.log
        log.info(s"[CalVer] Version: ${calVer.value}")
        log.info(s"[CalVer] Pattern: ${calVerPattern.value}")
      },
      calVerWriteVersionToFile := {
        val outputFile = calVerOutputFile.value.getOrElse(DefaultOutputFile)
        Files.write(
          Paths.get(outputFile),
          calVer.value.getBytes(StandardCharsets.UTF_8)
        )
      }
    )
  }

  // Automatically exposed globally.
  override lazy val globalSettings: Seq[Setting[_]] = calVerDefaults
}
